package ShahJava;

import javax.swing.*;
import java.io.*;
import javax.swing.filechooser.*;

/**
 * Class Name:  ImageFileView
 * Class Desc:  purpose of this class is to provide a customize
 *              image preview in the file dialog based on which file you choose.
 *              Heavily based on JFileChooser Tutorial in JavaSoft
 *
 *
 * Startdate:   04/29/1999
 *
 * @author:     Shah Mumin
 *
 * @version:    1.0         04/29/1999
 *
 */
public class ImageFileView extends FileView {

   //create imageicon to show in front of the file name
   ImageIcon jpgIcon = new ImageIcon("jpgIcon.gif");
   ImageIcon gifIcon = new ImageIcon("gifIcon.gif");
   ImageIcon tiffIcon = new ImageIcon("tiffIcon.gif");

   /**
    ************************************************************************
    * Each L&F JFileChooserUI object implements this class to pass back the correct
    * icons and type descriptions specific to that L&F.  JFileChooser first look
    * to see if there is a user-defined FileView, if there is, it gets type
    * information from there first.  Since this class returns icons for all image
    * files, it will returns icons for the image files and null for the other.
    *************************************************************************
    */



   /**
    * Method Name:  getName() (String method)
    * Method Desc:  You must provide this mehtod since there is no default
    *               implementation.
    *               Provides the name of the file. Normally this would be simply f.getName()
    *
    * @return       null   let the L&F FileView figure this out
    *
    */
   public String getName(File f) {
        return null;
   }

   /**
    * Method Name:  getDescription() (String method)
    * Method Desc:  You must provide this mehtod since there is no default
    *               implementation.
    *               Provides a human readable description of the file. For example
    *               a file named jag.jpg might have a description that read:
    *               "A JPEG image file of Shah Mumin's feet"
    *
    * @return       null   let the L&F FileView figure this out
    *
    */
   public String getDescription(File f) {
        return null;
   }


   /**
    * Method Name:  isTraversable() (Boolean method)
    * Method Desc:  You must provide this mehtod since there is no default
    *               implementation.
    *               Provides whether the directory is traversable or not. This might be
    *               useful, for example, if you want a directory to represent
    *               a compound document and don't want the user to descend into it.
    *
    * @return       null   let the L&F FileView figure this out
    *
    */
   public Boolean isTraversable(File f) {
        return null;
   }


   /**
    * Method Name:  getDescription() (String method)
    * Method Desc:  You must provide this mehtod since there is no default
    *               implementation.
    *               Provides a A human readable description of the type of the file. For
    *               Example, a jpg file might have a type description of:
    *               "A JPEG Compressed Image File"
    *
    * @return       type   Description of the image file
    *
    */
   public String getTypeDescription(File f) {

        String extension = getExtension(f);
        String type = null;

        if (extension != null) {
            if (extension.equals("jpeg") || extension.equals("jpg")) {
                type = "JPEG Image";
            }//end if
            else if (extension.equals("gif")){
                type = "GIF Image";
            } //end if
            else if (extension.equals("tiff") || extension.equals("tif")) {
                type = "TIFF Image";
            } //end if
        }
        //end if extention

        return type;

   } //end of getTypeDescription

   /**
    * Method Name:  getIcon() (String method)
    * Method Desc:  You must provide this mehtod since there is no default
    *               implementation.
    *               Provides a icon that represents this file in the JFileChooser.
    *
    * @param        f     the file itself
    * @return       icon   JImageIcon
    *
    */
   public Icon getIcon(File f) {

        String extension = getExtension(f);
        Icon icon = null;

        if (extension != null) {
            if (extension.equals("jpeg") || extension.equals("jpg")) {
                icon = jpgIcon;
            } //end if
            else if (extension.equals("gif")) {
                icon = gifIcon;
            } //end if
            else if (extension.equals("tiff") || extension.equals("tif")) {
                icon = tiffIcon;
            } //end if
        } //end if extension

        return icon;

   } //end of getIcon

   /**
    * Method Name:  getExtension() (String method)
    * Method Desc:  You must provide this mehtod since there is no default
    *               implementation.
    *               Gets the extension of this file. Code is factored out
    *               because we use this in both getIcon and getTypeDescription
    *
    * @param        f     the file itself
    * @return       ext   extention of the files
    *
    */
   private String getExtension(File f) {

        String ext = null;
        String s = f.getName();
        int i = s.lastIndexOf('.');

        if (i > 0 &&  i < s.length() - 1) {
            ext = s.substring(i+1).toLowerCase();
        } //end if

        return ext;

   }//end of getExtension

} //end of class ImageFileView
