package ShahJava;

import javax.swing.*;
import java.beans .*;
import java.io.File;
import java.awt.*;

/**
 * Class Name:  ImagePreview
 * Class Desc:  purpose of this class is to provide a customize
 *              image preview in the file dialog based on which file you choose.
 *              Heavily based on JFileChooser Tutorial in JavaSoft
 *
 *
 * Startdate:   04/29/1999
 *
 * @author:     Shah Mumin
 *
 * @version:    1.0         04/29/1999
 *
 */
class ImagePreview extends JComponent
                          implements PropertyChangeListener {

    ImageIcon thumbnail = null;
    File file = null;

   /**
    ************************************************************************
    * Aside from the previewer, probably the msot common use for the accessory
    * component is a panel with more controls on it -- say, checkboxes that
    * toggles some features.
    * Any object that inherits from JComponent can be an accessory component.
    * The component should have a prefered size that looks good in the file
    * chooser.
    *************************************************************************
    */



   /**
    * Description: Constructor of the class.  Initialtes the class ImagePreview.
    *
    * @param       fc     Recieves file chooser in the constructor and sets the
    *                     PropertyChangeListener(java.beans)
    *
    */
   public ImagePreview(JFileChooser fc) {

        //setting the prefered size
        setPreferredSize(new Dimension(100, 50));
        //setting the listener
        fc.addPropertyChangeListener(this);

   }//end of ImagePreview constructor



   /**
    * Method Name:  loadImage() (void method)
    * Method Desc:  Creates an ImageIcon to be shown on the file dialog
    *               as a preview.
    *
    */
   public void loadImage() {

        if (file == null)
            return;

        ImageIcon tmpIcon = new ImageIcon(file.getPath());
        if (tmpIcon.getIconWidth() > 90) {
            thumbnail = new ImageIcon(tmpIcon.getImage().
                                      getScaledInstance(90, -1,
                                                        Image.SCALE_DEFAULT));
        }
        else {
            thumbnail = tmpIcon;
        }

   } //end of loadImage


   /**
    * Method Name:  propertyChange() (void method)
    * Method Desc:  Acts when you select a different file
    *               Picks up the PropertyChangeEvent
    *
    * @param        e    PropertyChangeEvent
    *                    This method gets called when a bound property is changed.
    *                    PropertyChangeEvent object describing the event source
    *                    and the property that has changed.
    *
    */
   public void propertyChange(PropertyChangeEvent e) {

        String prop = e.getPropertyName();

        //Check whether the propertychange was a file chagne event
        //The method gets the file object from the file chooser
        if (prop.equals(JFileChooser.SELECTED_FILE_CHANGED_PROPERTY)) {
            file = (File) e.getNewValue();
            if (isShowing()) {
                //call the method to get image or null
                loadImage();
                //must repaint the screen
                repaint();
            }//end if
        }//end if

   }//end of propertyChange

   /**
    * Method Name:  paintComponent() (void method)
    * Method Desc:  Paints the thumbnail image to the FileDialog
    *
    */
   public void paintComponent(Graphics g) {

        if (thumbnail == null) {
            loadImage();
        }//end if

        if (thumbnail != null) {
            int x = getWidth()/2 - thumbnail.getIconWidth()/2;
            int y = getHeight()/2 - thumbnail.getIconHeight()/2;

            if (y < 0) {
                y = 0;
            } //end if

            if (x < 5) {
                x = 5;
            } //end if
            thumbnail.paintIcon(this, g, x, y);
        }  //end if

   } //end of paintComponent

} //end of class ImagePreview
