package ShahJava;

import javax.swing.filechooser.*;
import java.io.File;

public class OnlyOBJFiltering extends FileFilter {

    //We want to see only the following kind of files
    final static String obj = "obj";

    /**
     ************************************************************************
     * By default, a file chooser shows all of the files files and directories that
     * it detects, except hidden files.  The file chooser is calling the filter's
     * accept method for each file to determine whether it should be dispalyed.
     * You can filter based on file type, size, ownership,
     * etc.
     * Build-in filtering: all hidden files are filtered by default.
     *                call Call setFileHidingEnabled(false) to show hidden files.

     * Application-controlled filtering: Create a custom subclass of
     *                 FileFilter, instantiate it, and use the instance
     *                 as an argument to setFileFilter.  The file chooser
     *                 will show only those files that the filter accepts.
     *
     * User-choosable filtering:  What this class would do.
     *************************************************************************
     */



   /**
    * Method Name:  accept() (boolean method)
    * Method Desc:  You must provide this mehtod since there is no default
    *               implementation.
    *               Determines whether the given file is accepted by this filter.
    *
    * @param        f    File type  only obj files in this class
    * @return       boolean true or false
    *
    */
   public boolean accept(File f) {

        //Accepts all directories.  By accepting all directories, this filter
        //allows the user to navigate around the file system, otherwise, user would
        //be limited to the directory with which the chooser was initiated
        if (f.isDirectory()) {
            return true;
        }

        //The chooser will accept the files with the extentions defined below
        //all other files would not be shown
        String s = f.getName();
        int i = s.lastIndexOf('.');

        if (i > 0 &&  i < s.length() - 1) {
            String extension = s.substring(i+1).toLowerCase();

            if (obj.equals(extension)) {
                    return true;
            }
            else {
              return false;
            } //end if

        }//end if

        return false;
    } //end of accept

   /**
    * Method Name:  getDescription() (String method)
    * Method Desc:  You must provide this mehtod since there is no default
    *               implementation.
    *               provides The description of this filter.
    *
    * @return       a String containing your description.
    *
    */
   public String getDescription() {

        //This is what you will see in File Description
        return "OBJ Files";

   }
}


