package fr.cnrs.aniseed;

import java.awt.*;
import javax.swing.*;
import javax.swing.tree.*;
import java.awt.event.*;

/**
 * @author Olivier Tassy
 *
 * @version 1.0, Created on 11 juillet 2003, 12:42
 */

public class DynamicTree extends JPanel {

    static public anatomyPart rootNode;
    protected DefaultTreeModel treeModel;
    protected JTree tree;
    private Toolkit toolkit = Toolkit.getDefaultToolkit();

////////////////////////////////////////////////////////////////////////////////////////////////////
//Constructor
    public DynamicTree() {

        rootNode=new anatomyPart("Whole embryo","organism");
        treeModel = new DefaultTreeModel(rootNode);

        tree = new JTree(treeModel);

        tree.setEditable(false);
        tree.getSelectionModel().setSelectionMode(TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
        tree.setShowsRootHandles(true);
        tree.setCellRenderer(new MyRenderer());

        tree.addMouseListener(new NodeSelectionListener(tree));

        JScrollPane scrollPane = new JScrollPane(tree);
        setLayout(new GridLayout(1,0));
        add(scrollPane);
    }
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
    class NodeSelectionListener extends MouseAdapter {
        JTree tree;
////////////////////////////////////////////////////////////////////////////////////////////////////
        NodeSelectionListener(JTree tree) {
            this.tree = tree;
        }
////////////////////////////////////////////////////////////////////////////////////////////////////
        public void mouseClicked(MouseEvent e) {

            if(e.getClickCount()==2){
                launchBrowser();
                return;
            }


            if( (! e.isControlDown())&&(! e.isShiftDown())&&(e.getClickCount()==1) ){
                int x = e.getX();
                int y = e.getY();
                int row = tree.getRowForLocation(x, y);
                TreePath  path = tree.getPathForRow(row);

                if (path != null) {
                    anatomyPart node = (anatomyPart)path.getLastPathComponent();
                    boolean isSelected = ! (node.isSelected());
                    node.setSelected(isSelected);

                    tree.expandPath(path);

                    ((DefaultTreeModel)tree.getModel()).nodeChanged(node);
                    // I need revalidate if node is root.  but why?
                    if (row == 0) {
                        tree.revalidate();
                        tree.repaint();
                    }
                }
            }
        }
////////////////////////////////////////////////////////////////////////////////////////////////////
    }
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
    public void launchBrowser(){

        try{
            anatomyPart part = this.getSelected();
            if(part != null){
                if( part.getTitle().equals("gene") ){
                    BrowserLauncher.openURL( VirtualEmbryo.url+"molecule-gene.php?name="+part.getName() );
                }
                if( part.getTitle().equals("cdna") ){
                    BrowserLauncher.openURL( VirtualEmbryo.url+"molecule-clone.php?name="+part.getName() );
                }
                if( part.getTitle().equals("clone") ){
                    BrowserLauncher.openURL( VirtualEmbryo.url+"molecule-clone2.php?name="+part.getName() );
                }
                if( part.getTitle().equals("is") ){
                    BrowserLauncher.openURL( VirtualEmbryo.url+"insitu.php?id="+part.getName() );
                }
                if( part.getTitle().equals("cell") ){
                    BrowserLauncher.openURL( VirtualEmbryo.url+"short_hierarchy.php?region="+part.getID()+"&dev="+VirtualEmbryo.devstage );
                }
                if( part.getTitle().equals("region") ){
                    BrowserLauncher.openURL( VirtualEmbryo.url+"short_hierarchy.php?region="+part.getID()+"&dev="+VirtualEmbryo.devstage );
                }
                if( part.getTitle().equals("structure") ){
                    BrowserLauncher.openURL( VirtualEmbryo.url+"short_hierarchy.php?region="+part.getID()+"&dev="+VirtualEmbryo.devstage );
                }
                if( part.getTitle().equals("organism") ){
                    BrowserLauncher.openURL( VirtualEmbryo.url+"short_hierarchy.php?region="+part.getID()+"&dev="+VirtualEmbryo.devstage );
                }
                if( part.getTitle().equals("folder") ){
                    BrowserLauncher.openURL( VirtualEmbryo.url+"molecule-gene.php?name="+part.getID() );
                }

            }
        }catch(Exception e){;}

    }
////////////////////////////////////////////////////////////////////////////////////////////////////
    public void expandMainObject(String mode){
        collapseAllObject();
        int row = 0;
        while (row < tree.getRowCount()) {
            if( tree.getPathForRow(row).getPathCount()<3 )
                tree.expandRow(row);

            row++;
        }
    }
////////////////////////////////////////////////////////////////////////////////////////////////////
    public void expandAllObject(){
        int row = 0;
        while (row < tree.getRowCount()) {
            tree.expandRow(row);
            row++;
        }
    }
////////////////////////////////////////////////////////////////////////////////////////////////////
    public void collapseAllObject(){
        int row = tree.getRowCount() - 1;
        while (row >= 0) {
            tree.collapseRow(row);
            row--;
        }
    }
////////////////////////////////////////////////////////////////////////////////////////////////////
//Expand the currently selected node
    public void expandObject() {
        tree.expandPath( tree.getSelectionPath() );
    }
////////////////////////////////////////////////////////////////////////////////////////////////////
//Collapse tree
    public void collapseObject() {
        tree.collapseRow( tree.getRowForPath(tree.getSelectionPath()) );
    }
////////////////////////////////////////////////////////////////////////////////////////////////////
    public void setRoot(String name, String id) {
        rootNode.setID(id);
        rootNode.setName(name);
    }
////////////////////////////////////////////////////////////////////////////////////////////////////
    public void setRoot(String name, String id, String type, cell shape) {
        rootNode.setID(id);
        rootNode.setName(name);
        rootNode.setTitle(type);
        rootNode.setShape(shape);
    }
////////////////////////////////////////////////////////////////////////////////////////////////////
// Add child to the currently selected node.
    public anatomyPart addObject(Object child, String type, String ID) {
        anatomyPart parentNode = null;
        TreePath parentPath = tree.getSelectionPath();

        if (parentPath == null) {
            parentNode = rootNode;
        } else {
            parentNode = (anatomyPart) (parentPath.getLastPathComponent());
        }
        return addObject(parentNode, child, type, ID, null, true);
    }
////////////////////////////////////////////////////////////////////////////////////////////////////
    public anatomyPart addObject(anatomyPart parent,Object child, String type, String ID, cell shape) {
        return addObject(parent, child, type, ID, shape, false);
    }
////////////////////////////////////////////////////////////////////////////////////////////////////
    public anatomyPart addObject(anatomyPart parent,Object child,String type, String ID, cell shape, boolean shouldBeVisible) {
        anatomyPart childNode= new anatomyPart(child,type,ID,shape);
         if (parent == null) {
            parent = rootNode;
        }

        if ( (parent.getTitle().equals("organism"))&&(childNode.getTitle().equals("region")) )
            childNode.setTitle("structure");

        if(!type.equals("invalid"))
            treeModel.insertNodeInto( childNode, parent, parent.getChildCount() );

        if (shouldBeVisible) {
            tree.scrollPathToVisible(new TreePath(childNode.getPath()));
        }
        return childNode;
    }
    ////////////////////////////////////////////////////////////////////////////////////////////////////
    public anatomyPart getrootNode(){
        return rootNode;
    }
    ////////////////////////////////////////////////////////////////////////////////////////////////////
    public anatomyPart getSelected(){
        try{
            if(tree.getSelectionPath().getLastPathComponent().toString().equals(""))
                return null;
            else
                return (anatomyPart) tree.getSelectionPath().getLastPathComponent();
        }catch(Exception e){return null;}
    }
    ////////////////////////////////////////////////////////////////////////////////////////////////////
    public JTree getTree(){
        return tree;
    }
////////////////////////////////////////////////////////////////////////////////////////////////////
    public void update(){
        tree.updateUI();
    }
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
    public class MyRenderer extends DefaultTreeCellRenderer implements TreeCellRenderer {
        protected JCheckBox check;
        protected TreeLabel label;
////////////////////////////////////////////////////////////////////////////////////////////////////
//Constructor
        public MyRenderer() {
            setLayout(null);
            add(check = new JCheckBox());
            add(label = new TreeLabel());

        }
////////////////////////////////////////////////////////////////////////////////////////////////////
        public Component getTreeCellRendererComponent(JTree tree, Object value, boolean isSelected, boolean expanded, boolean leaf, int row, boolean hasFocus) {
            String stringValue = tree.convertValueToText(value, isSelected, expanded, leaf, row, hasFocus);
            setEnabled(tree.isEnabled());

            anatomyPart part=(anatomyPart)value;

            check.setSelected(part.isSelected());

//            try{
//                //this allows to interact only with loaded 3d objects
//                if( (VirtualEmbryo.getObj().get( part.getName() )==null)&&(!part.getTitle().equals("is"))  )
//                    check.setSelected(false);
//            }catch(Exception e){;}

            label.setFont(tree.getFont());
            label.setText(stringValue);
            label.setSelected(isSelected);
            label.setFocus(hasFocus);

            try{
                if( (VirtualEmbryo.getObj().get( part.getName() )==null)&&(!part.getTitle().equals("is"))  )
                    part.setColor(new Color(255,255,255));
            }catch(Exception e){;}

            check.setBackground( part.getColor() );

            String imgPath = "resources/img/";

            if(part.getTitle().equals("organism") ){
                label.setIcon(new javax.swing.ImageIcon(getClass().getResource(imgPath+"tetard.gif")));
            }
            else if (part.getTitle().equals("structure")) {
                label.setIcon(new javax.swing.ImageIcon(getClass().getResource(imgPath+"structure.jpg")));
            }
            else if (part.getTitle().equals("region")) {
                label.setIcon(new javax.swing.ImageIcon(getClass().getResource(imgPath+"region.gif")));
            }
            else if (part.getTitle().equals("cell")) {
                label.setIcon(new javax.swing.ImageIcon(getClass().getResource(imgPath+"cell.gif")));
            }
            else if (part.getTitle().equals("gene")) {
                label.setIcon(new javax.swing.ImageIcon(getClass().getResource(imgPath+"dna.gif")));
            }
            else if (part.getTitle().equals("is")) {
                label.setIcon(new javax.swing.ImageIcon(getClass().getResource(imgPath+"insitu.gif")));
            }
            else if (part.getTitle().equals("cdna")) {
                label.setIcon(new javax.swing.ImageIcon(getClass().getResource(imgPath+"cdna.gif")));
            }
            else if (part.getTitle().equals("clone")) {
                label.setIcon(new javax.swing.ImageIcon(getClass().getResource(imgPath+"cdna.gif")));
            }
            else if (part.getTitle().equals("fate")) {
                label.setIcon(new javax.swing.ImageIcon(getClass().getResource(imgPath+"fate.gif")));
            }
            else if (part.getTitle().equals("folder")) {
                label.setIcon(new javax.swing.ImageIcon(getClass().getResource(imgPath+"stage.gif")));
            }
            else{ setIcon(null); }

            this.doLayout();
            return this;
        }
////////////////////////////////////////////////////////////////////////////////////////////////////
        public Dimension getPreferredSize() {
            Dimension d_check = check.getPreferredSize();
            Dimension d_label = label.getPreferredSize();
            return new Dimension(d_check.width  + d_label.width,
            (d_check.height < d_label.height ?
            d_label.height : d_check.height));
        }
////////////////////////////////////////////////////////////////////////////////////////////////////
        public void doLayout() {
            Dimension d_check = check.getPreferredSize();
            Dimension d_label = label.getPreferredSize();
            int y_check = 0;
            int y_label = 0;
            if (d_check.height < d_label.height) {
                y_check = (d_label.height - d_check.height)/2;
            } else {
                y_label = (d_check.height - d_label.height)/2;
            }
            check.setLocation(0,y_check);
            check.setBounds(0,y_check,d_check.width,d_check.height);
            label.setLocation(d_check.width,y_label);
            label.setBounds(d_check.width,y_label,d_label.width,d_label.height);
        }
////////////////////////////////////////////////////////////////////////////////////////////////////
        public void setBackground(Color color) {
            if (color instanceof javax.swing.plaf.ColorUIResource)
                color = null;
            super.setBackground(color);
        }
////////////////////////////////////////////////////////////////////////////////////////////////////
        class TreeLabel extends JLabel {
            boolean isSelected;
            boolean hasFocus;
////////////////////////////////////////////////////////////////////////////////////////////////////
            TreeLabel() {
            }

            public void setBackground(Color color) {
                if(color instanceof javax.swing.plaf.ColorUIResource)
                    color = null;
                super.setBackground(color);
            }

            public void paint(Graphics g) {
                String str;
                if ((str = getText()) != null) {
                    if (0 < str.length()) {
                        if (isSelected) {
                            g.setColor(UIManager.getColor("Tree.selectionBackground"));
                        } else {
                            g.setColor(UIManager.getColor("Tree.textBackground"));
                        }
                        Dimension d = getPreferredSize();
                        int imageOffset = 0;
                        Icon currentI = getIcon();
                        if (currentI != null) {
                            imageOffset = currentI.getIconWidth() + Math.max(0, getIconTextGap() - 1);
                        }
                        g.fillRect(imageOffset, 0, d.width -1 - imageOffset, d.height);
                        if (hasFocus) {
                            g.setColor(UIManager.getColor("Tree.selectionBorderColor"));
                            g.drawRect(imageOffset, 0, d.width -1 - imageOffset, d.height -1);
                        }
                    }
                }
                super.paint(g);
            }

            public Dimension getPreferredSize() {
                Dimension retDimension = super.getPreferredSize();
                if (retDimension != null) {
                    retDimension = new Dimension(retDimension.width + 3,
                    retDimension.height);
                }
                return retDimension;
            }

            void setSelected(boolean isSelected) {
                this.isSelected = isSelected;
            }

            boolean isSelected() {
                return isSelected;
            }

            void setFocus(boolean hasFocus) {
                this.hasFocus = hasFocus;
            }
        }
    }
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
    public void destroy(){
        try{
            tree.removeAll();
            tree=null;
        }catch(Exception e){;}
    }
////////////////////////////////////////////////////////////////////////////////////////////////////
}

