package fr.cnrs.aniseed;

import javax.swing.*;
import java.io.*;
import java.awt.Color;

import java.util.*;
import java.sql.*;
import java.util.regex.*;
import java.net.URL;

import ShahJava.*;

/**
 * Copyright 2003-2013 Olivier Tassy/Patrick Lemaire CNRS Montpellier
 * 3D Virtual Embryo program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * 3D Virtual Embryo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * @author Olivier Tassy
 * @contributor Cyril Martin
 *
 * @version 1.1.1, Modified on 15 june 2011, 12:11
 * @version 1.2, Modified on 17 oct. 2012, 16:51
 * @version 1.3, Modified on 1 aug. 2013, 11:25 (compatible with new address "/v3" since v4 is online)
 */

//Main class
public class VirtualEmbryo extends JFrame {

    // --------------------------------------
    //Please complete the following variables with your own database parameters
    // --------------------------------------
    static final String serverIP="194.57.116.102";
    static final String dataBase="aniseed3";


    //Database connection
    static final String database="jdbc:postgresql://"+serverIP+"/"+dataBase;

    static final String DBlog="aniseed_public";
    static final String DBpass="lemaire";


    //Public site URL
    static final String url="http://www.aniseed.cnrs.fr/v3/";

    private Connection con; //JDBC Connection

    static Hashtable lineagelist; //save selected cells when user switch to next or previous stage to stain afiliated cells

    static javax.media.j3d.Transform3D view; //save the last embryo point of view
    int NavEmbryoPos; //allows to know which embryo to load when the user click the next or previous button
    static embryo embryon;
    static DynamicTree treePanel = new DynamicTree();

    static AppList applist=new AppList(); //store selected cells appearance in order to make update

    static String species=new String();
    static String devstage=new String();

    static setMaterial materialPanel=new setMaterial();

    String lastPath=new String();
    static String lastPicturePath=new String();

////////////////////////////////////////////////////////////////////////////////////////////////////
//Constructor
    public VirtualEmbryo() {

    try{
        UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
    } catch (Exception e) {System.out.println("error");}

        //Compatibility between swing and Java3D
        JPopupMenu.setDefaultLightWeightPopupEnabled(false);
        ToolTipManager.sharedInstance().setLightWeightPopupEnabled(false);

        ToolTipManager.sharedInstance().setDismissDelay(60000);
        setIconImage(new ImageIcon(this.getClass().getResource("resources/img/3dve.gif")).getImage());
        initComponents();

        //Load embryolist
        embryolist("embryo/", "embryo-list.txt");

        this.setSize(800,550);

        JOptionPane diag2=new JOptionPane();
        diag2.setBackground(new Color(255,255,255));
        diag2.setOptionType(JOptionPane.INFORMATION_MESSAGE);
        diag2.showMessageDialog(new javax.swing.JFrame(),"3D Virtual Embryo program is free software; you can redistribute it and/or\nmodify it under the terms of the GNU Lesser General Public\nLicense as published by the Free Software Foundation; either\nversion 2.1 of the License, or any later version.\n\n3D Virtual Embryo is distributed in the hope that it will be useful,\nbut WITHOUT ANY WARRANTY; without even the implied warranty of\nMERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU\nLesser General Public License in the Help menu for more details.\n\nCopyright © 2003-2013 Olivier Tassy/Patrick Lemaire CNRS Montpellier");
    }
////////////////////////////////////////////////////////////////////////////////////////////////////
//Retrieve and Load a file containing a list of embryos
    public void embryolist(String path, String listfile){

        //find 3D embryos available//
        try{
            EmbryoJMenu.removeAll();
        }catch(Exception e){;}
        try{
            /* local launch */
            // BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(path+File.separatorChar+listfile)));

            /* jnlp launch */
            URL embryo_url = new URL(url + "3dve/embryo/embryo-list.txt");
            BufferedReader br = new BufferedReader(new InputStreamReader(embryo_url.openStream()));

            String line = "";
            EmbryoMenu embryomenu;
            int count=0;
            while((line = br.readLine()) != null) {
                String[] info=line.split("\t");

                //0=>name 1=>file 2=>stage_id
                try{
                    embryomenu=new EmbryoMenu(info[0],path+info[1],info[2],count++);
                }catch(Exception e){
                    embryomenu=new EmbryoMenu(info[0],path+info[1],count++);
                }

                embryomenu.addActionListener(new java.awt.event.ActionListener() {
                    public void actionPerformed(java.awt.event.ActionEvent evt) {
                        loadEmbryo(evt);
                    }
                });

                EmbryoJMenu.add( embryomenu );

            }
            br.close();
        }catch(Exception e) {
            ;
            JOptionPane diag2=new JOptionPane();
            diag2.showMessageDialog(new javax.swing.JFrame(),"Error: "+ e.getMessage());
        }

    }
////////////////////////////////////////////////////////////////////////////////////////////////////
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        buttonGroup1 = new javax.swing.ButtonGroup();
        buttonGroup2 = new javax.swing.ButtonGroup();
        buttonGroup3 = new javax.swing.ButtonGroup();
        buttonGroup4 = new javax.swing.ButtonGroup();
        jSplitPane1 = new javax.swing.JSplitPane();
        CanvasPanel = new javax.swing.JPanel();
        javax.swing.JPanel jPanel2 = new javax.swing.JPanel();
        javax.swing.JTabbedPane jTabbedPane1 = new javax.swing.JTabbedPane();
        javax.swing.JTabbedPane jTabbedPane4 = new javax.swing.JTabbedPane();
        javax.swing.JPanel jPanel5 = new javax.swing.JPanel();
        ScrollPaneSelection = new javax.swing.JScrollPane();
        javax.swing.JPanel jPanel1 = new javax.swing.JPanel();
        javax.swing.JPanel jPanel3 = new javax.swing.JPanel();
        javax.swing.JButton checkinvert = new javax.swing.JButton();
        javax.swing.JButton expall = new javax.swing.JButton();
        javax.swing.JButton check = new javax.swing.JButton();
        javax.swing.JButton collall = new javax.swing.JButton();
        javax.swing.JButton jButton1 = new javax.swing.JButton();
        javax.swing.JButton jButton7 = new javax.swing.JButton();
        javax.swing.JButton selectchecked = new javax.swing.JButton();
        javax.swing.JButton selectinvert = new javax.swing.JButton();
        javax.swing.JButton checkall = new javax.swing.JButton();
        javax.swing.JButton checknone = new javax.swing.JButton();
        ScrollPaneSlicer = new javax.swing.JScrollPane();
        jPanel4 = new javax.swing.JPanel();
        ScrollPaneSelection1 = new javax.swing.JScrollPane();
        txt = new javax.swing.JTextArea();
        javax.swing.JPanel jPanel11 = new javax.swing.JPanel();
        javax.swing.JPanel jPanel31 = new javax.swing.JPanel();
        javax.swing.JButton jButton71 = new javax.swing.JButton();
        javax.swing.JButton selectchecked1 = new javax.swing.JButton();
        jButton15 = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        hiera = new javax.swing.JCheckBox();
        javax.swing.JTabbedPane jTabbedPane2 = new javax.swing.JTabbedPane();
        ScrollPaneSurf = new javax.swing.JScrollPane();
        jScrollPaneNeighbor = new javax.swing.JScrollPane();
        jScrollPaneDistance = new javax.swing.JScrollPane();
        jScrollPaneSphericity = new javax.swing.JScrollPane();
        jTabbedPane3 = new javax.swing.JTabbedPane();
        ScrollPaneQuery = new javax.swing.JScrollPane();
        ScrollPaneQuery2 = new javax.swing.JScrollPane();
        jTabbedPane5 = new javax.swing.JTabbedPane();
        jScrollPaneFate = new javax.swing.JScrollPane();
        ScrollPaneQuery3 = new javax.swing.JScrollPane();
        jScrollPane1 = new javax.swing.JScrollPane();
        jScrollPane2 = new javax.swing.JScrollPane();
        javax.swing.JToolBar jToolBar1 = new javax.swing.JToolBar();
        jButtonprev = new javax.swing.JButton();
        jButtonnext = new javax.swing.JButton();
        jButton13 = new javax.swing.JButton();
        jButton11 = new javax.swing.JButton();
        jButton12 = new javax.swing.JButton();
        javax.swing.JButton jButton4 = new javax.swing.JButton();
        javax.swing.JButton jButton10 = new javax.swing.JButton();
        javax.swing.JButton jButtonTransparency = new javax.swing.JButton();
        javax.swing.JLabel jLabel21 = new javax.swing.JLabel();
        javax.swing.JButton jButton8 = new javax.swing.JButton();
        javax.swing.JButton jButton9 = new javax.swing.JButton();
        javax.swing.JButton ButtonSnapshot = new javax.swing.JButton();
        javax.swing.JLabel jLabel2 = new javax.swing.JLabel();
        javax.swing.JButton jButton3 = new javax.swing.JButton();
        javax.swing.JButton jButton5 = new javax.swing.JButton();
        javax.swing.JButton jButton6 = new javax.swing.JButton();
        javax.swing.JButton jButton2 = new javax.swing.JButton();
        menuBar = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem ChangeList = new javax.swing.JMenuItem();
        javax.swing.JMenuItem openMenuItem = new javax.swing.JMenuItem();
        javax.swing.JMenuItem exitMenuItem = new javax.swing.JMenuItem();
        EmbryoJMenu = new javax.swing.JMenu();
        AppearanceMenu = new javax.swing.JMenu();
        MaterialMenuItem = new javax.swing.JMenuItem();
        jMenu1 = new javax.swing.JMenu();
        jRadioButtonMenuItem12 = new javax.swing.JRadioButtonMenuItem();
        jRadioButtonMenuItem1 = new javax.swing.JRadioButtonMenuItem();
        jRadioButtonMenuItem13 = new javax.swing.JRadioButtonMenuItem();
        aliasingRadio = new javax.swing.JRadioButtonMenuItem();
        toolsMenu = new javax.swing.JMenu();
        MenuItemSnapshot = new javax.swing.JMenuItem();
        MenuItemResetView = new javax.swing.JMenuItem();
        tripodRadioButton = new javax.swing.JRadioButtonMenuItem();
        helpMenu = new javax.swing.JMenu();
        contentsMenuItem = new javax.swing.JMenuItem();
        aboutMenuItem = new javax.swing.JMenuItem();

        setTitle("3D Virtual Embryo");
        setBackground(new java.awt.Color(255, 255, 255));
        setFont(new java.awt.Font("Arial", 0, 10));
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        jSplitPane1.setDividerSize(5);
        jSplitPane1.setContinuousLayout(true);
        jSplitPane1.setFont(new java.awt.Font("Arial", 0, 12));
        jSplitPane1.setMinimumSize(new java.awt.Dimension(678, 73));

        CanvasPanel.setFont(new java.awt.Font("Arial", 0, 12));
        CanvasPanel.setMinimumSize(new java.awt.Dimension(0, 0));
        CanvasPanel.setPreferredSize(new java.awt.Dimension(700, 500));
        CanvasPanel.setLayout(new java.awt.BorderLayout());
        jSplitPane1.setLeftComponent(CanvasPanel);

        jPanel2.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanel2.setFont(new java.awt.Font("Arial", 0, 12));
        jPanel2.setMinimumSize(new java.awt.Dimension(200, 100));
        jPanel2.setPreferredSize(new java.awt.Dimension(0, 0));
        jPanel2.setLayout(new java.awt.BorderLayout());

        jTabbedPane1.setBackground(new java.awt.Color(153, 0, 0));
        jTabbedPane1.setForeground(new java.awt.Color(255, 255, 255));
        jTabbedPane1.setTabPlacement(javax.swing.JTabbedPane.BOTTOM);
        jTabbedPane1.setFont(new java.awt.Font("Arial", 3, 12));
        jTabbedPane1.setMinimumSize(new java.awt.Dimension(96, 67));
        jTabbedPane1.setPreferredSize(new java.awt.Dimension(235, 67));

        jPanel5.setFont(new java.awt.Font("Arial", 0, 12));
        jPanel5.setLayout(new java.awt.BorderLayout());

        ScrollPaneSelection.setFont(new java.awt.Font("Arial", 0, 12));
        jPanel5.add(ScrollPaneSelection, java.awt.BorderLayout.CENTER);

        jPanel1.setFont(new java.awt.Font("Arial", 0, 12));

        jPanel3.setFont(new java.awt.Font("Arial", 0, 12));
        jPanel3.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        checkinvert.setBackground(new java.awt.Color(153, 0, 0));
        checkinvert.setFont(new java.awt.Font("Arial", 1, 9));
        checkinvert.setForeground(new java.awt.Color(255, 255, 255));
        checkinvert.setText("Tick invert");
        checkinvert.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                checkinvertActionPerformed(evt);
            }
        });
        jPanel3.add(checkinvert, new org.netbeans.lib.awtextra.AbsoluteConstraints(120, 40, 120, 20));

        expall.setBackground(new java.awt.Color(153, 0, 0));
        expall.setFont(new java.awt.Font("Arial", 1, 9));
        expall.setForeground(new java.awt.Color(255, 255, 255));
        expall.setText("Expand all");
        expall.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                expallActionPerformed(evt);
            }
        });
        jPanel3.add(expall, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 0, 120, 20));

        check.setBackground(new java.awt.Color(153, 0, 0));
        check.setFont(new java.awt.Font("Arial", 1, 9));
        check.setForeground(new java.awt.Color(255, 255, 255));
        check.setText("Tick selection");
        check.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                checkActionPerformed(evt);
            }
        });
        jPanel3.add(check, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 40, 120, 20));

        collall.setBackground(new java.awt.Color(153, 0, 0));
        collall.setFont(new java.awt.Font("Arial", 1, 9));
        collall.setForeground(new java.awt.Color(255, 255, 255));
        collall.setText("Collapse all");
        collall.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                collallActionPerformed(evt);
            }
        });
        jPanel3.add(collall, new org.netbeans.lib.awtextra.AbsoluteConstraints(120, 0, 120, 20));

        jButton1.setBackground(new java.awt.Color(153, 0, 0));
        jButton1.setFont(new java.awt.Font("Arial", 1, 9));
        jButton1.setForeground(new java.awt.Color(255, 255, 255));
        jButton1.setText("Color");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });
        jPanel3.add(jButton1, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 80, 120, 20));

        jButton7.setBackground(new java.awt.Color(153, 0, 0));
        jButton7.setFont(new java.awt.Font("Arial", 1, 9));
        jButton7.setForeground(new java.awt.Color(255, 255, 255));
        jButton7.setText("RESET");
        jButton7.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton7ActionPerformed(evt);
            }
        });
        jPanel3.add(jButton7, new org.netbeans.lib.awtextra.AbsoluteConstraints(120, 80, 120, 20));

        selectchecked.setBackground(new java.awt.Color(153, 0, 0));
        selectchecked.setFont(new java.awt.Font("Arial", 1, 9));
        selectchecked.setForeground(new java.awt.Color(255, 255, 255));
        selectchecked.setText("Select ticked");
        selectchecked.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                selectcheckedActionPerformed(evt);
            }
        });
        jPanel3.add(selectchecked, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 60, 120, 20));

        selectinvert.setBackground(new java.awt.Color(153, 0, 0));
        selectinvert.setFont(new java.awt.Font("Arial", 1, 9));
        selectinvert.setForeground(new java.awt.Color(255, 255, 255));
        selectinvert.setText("Select invert");
        selectinvert.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                selectinvertActionPerformed(evt);
            }
        });
        jPanel3.add(selectinvert, new org.netbeans.lib.awtextra.AbsoluteConstraints(120, 60, 120, 20));

        checkall.setBackground(new java.awt.Color(153, 0, 0));
        checkall.setFont(new java.awt.Font("Arial", 1, 9));
        checkall.setForeground(new java.awt.Color(255, 255, 255));
        checkall.setText("Check all");
        checkall.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                checkallActionPerformed(evt);
            }
        });
        jPanel3.add(checkall, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 20, 120, 20));

        checknone.setBackground(new java.awt.Color(153, 0, 0));
        checknone.setFont(new java.awt.Font("Arial", 1, 9));
        checknone.setForeground(new java.awt.Color(255, 255, 255));
        checknone.setText("Check none");
        checknone.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                checknoneActionPerformed(evt);
            }
        });
        jPanel3.add(checknone, new org.netbeans.lib.awtextra.AbsoluteConstraints(120, 20, 120, 20));

        jPanel1.add(jPanel3);

        jPanel5.add(jPanel1, java.awt.BorderLayout.SOUTH);

        jTabbedPane4.addTab("Anatomical dictionary", jPanel5);

        ScrollPaneSlicer.setFont(new java.awt.Font("Arial", 0, 12));
        jTabbedPane4.addTab("Slicer", ScrollPaneSlicer);

        jPanel4.setLayout(new java.awt.BorderLayout());

        ScrollPaneSelection1.setFont(new java.awt.Font("Arial", 0, 12));

        txt.setDragEnabled(true);
        ScrollPaneSelection1.setViewportView(txt);

        jPanel4.add(ScrollPaneSelection1, java.awt.BorderLayout.CENTER);

        jPanel11.setFont(new java.awt.Font("Arial", 0, 12));

        jPanel31.setFont(new java.awt.Font("Arial", 0, 12));
        jPanel31.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        jButton71.setBackground(new java.awt.Color(153, 0, 0));
        jButton71.setFont(new java.awt.Font("Arial", 1, 9));
        jButton71.setForeground(new java.awt.Color(255, 255, 255));
        jButton71.setText("RESET");
        jButton71.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton71ActionPerformed(evt);
            }
        });
        jPanel31.add(jButton71, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 80, 180, 20));

        selectchecked1.setBackground(new java.awt.Color(153, 0, 0));
        selectchecked1.setFont(new java.awt.Font("Arial", 1, 9));
        selectchecked1.setForeground(new java.awt.Color(255, 255, 255));
        selectchecked1.setText("Select");
        selectchecked1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                selectchecked1ActionPerformed(evt);
            }
        });
        jPanel31.add(selectchecked1, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 40, 130, 20));

        jButton15.setBackground(new java.awt.Color(153, 0, 0));
        jButton15.setFont(new java.awt.Font("Arial", 1, 12));
        jButton15.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton15ActionPerformed(evt);
            }
        });
        jPanel31.add(jButton15, new org.netbeans.lib.awtextra.AbsoluteConstraints(130, 40, 50, 20));

        jLabel1.setFont(new java.awt.Font("Arial", 1, 12));
        jLabel1.setText("<html><center>Copy/Paste your anatomical ID list from ANISEED</center></html>");
        jPanel31.add(jLabel1, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 0, 180, 40));

        hiera.setFont(new java.awt.Font("Arial", 1, 12));
        hiera.setSelected(true);
        hiera.setText("Use anatomical hierarchy");
        jPanel31.add(hiera, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 60, 180, 20));

        jPanel11.add(jPanel31);

        jPanel4.add(jPanel11, java.awt.BorderLayout.SOUTH);

        jTabbedPane4.addTab("Paste ID list", jPanel4);

        jTabbedPane1.addTab("Anatomy", jTabbedPane4);

        jTabbedPane2.setBackground(new java.awt.Color(153, 0, 0));
        jTabbedPane2.setForeground(new java.awt.Color(255, 255, 255));
        jTabbedPane2.setFont(new java.awt.Font("Arial", 1, 10));

        ScrollPaneSurf.setFont(new java.awt.Font("Arial", 0, 12));
        jTabbedPane2.addTab("Volume & surface", ScrollPaneSurf);

        jScrollPaneNeighbor.setFont(new java.awt.Font("Arial", 0, 12));
        jTabbedPane2.addTab("Neighbour & surface of contact", jScrollPaneNeighbor);

        jScrollPaneDistance.setFont(new java.awt.Font("Arial", 0, 12));
        jTabbedPane2.addTab("Distance    ", jScrollPaneDistance);
        jTabbedPane2.addTab("Cell geometry descriptor", jScrollPaneSphericity);

        jTabbedPane1.addTab("Biometry", jTabbedPane2);

        jTabbedPane3.setBackground(new java.awt.Color(153, 0, 0));
        jTabbedPane3.setForeground(new java.awt.Color(255, 255, 255));
        jTabbedPane3.setFont(new java.awt.Font("Arial", 1, 10));

        ScrollPaneQuery.setFont(new java.awt.Font("Arial", 0, 12));
        jTabbedPane3.addTab("From genes to structures", ScrollPaneQuery);

        ScrollPaneQuery2.setFont(new java.awt.Font("Arial", 0, 12));
        jTabbedPane3.addTab("From structures to genes", ScrollPaneQuery2);

        jTabbedPane1.addTab("Gene expression", jTabbedPane3);

        jScrollPaneFate.setFont(new java.awt.Font("Arial", 0, 12));
        jTabbedPane5.addTab("From fates to structures", jScrollPaneFate);

        ScrollPaneQuery3.setFont(new java.awt.Font("Arial", 0, 12));
        jTabbedPane5.addTab("From structures to fates", ScrollPaneQuery3);
        jTabbedPane5.addTab("Competence", jScrollPane1);
        jTabbedPane5.addTab("Induction", jScrollPane2);

        jTabbedPane1.addTab("Fate", jTabbedPane5);

        jPanel2.add(jTabbedPane1, java.awt.BorderLayout.CENTER);

        jSplitPane1.setRightComponent(jPanel2);

        getContentPane().add(jSplitPane1, java.awt.BorderLayout.CENTER);

        jToolBar1.setBackground(new java.awt.Color(153, 0, 0));
        jToolBar1.setFont(new java.awt.Font("Arial", 1, 12));

        jButtonprev.setBackground(new java.awt.Color(153, 0, 0));
        jButtonprev.setFont(new java.awt.Font("Arial", 1, 12));
        jButtonprev.setIcon(new javax.swing.ImageIcon(getClass().getResource("resources/img/back.jpg"))); // NOI18N
        jButtonprev.setToolTipText("prev. stage");
        jButtonprev.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonprevActionPerformed(evt);
            }
        });
        jToolBar1.add(jButtonprev);

        jButtonnext.setBackground(new java.awt.Color(153, 0, 0));
        jButtonnext.setIcon(new javax.swing.ImageIcon(getClass().getResource("resources/img/next.jpg"))); // NOI18N
        jButtonnext.setToolTipText("next stage");
        jButtonnext.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonnextActionPerformed(evt);
            }
        });
        jToolBar1.add(jButtonnext);

        jButton13.setBackground(new java.awt.Color(153, 0, 0));
        jButton13.setFont(new java.awt.Font("Arial", 1, 12));
        jButton13.setText("       ");
        jButton13.setBorder(null);
        jToolBar1.add(jButton13);

        jButton11.setBackground(new java.awt.Color(153, 0, 0));
        jButton11.setIcon(new javax.swing.ImageIcon(getClass().getResource("resources/img/annotation2.jpg"))); // NOI18N
        jButton11.setToolTipText("Annotation");
        jButton11.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton11ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton11);

        jButton12.setBackground(new java.awt.Color(153, 0, 0));
        jButton12.setIcon(new javax.swing.ImageIcon(getClass().getResource("resources/img/TextOnTop2.jpg"))); // NOI18N
        jButton12.setToolTipText("Text on top");
        jButton12.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton12ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton12);

        jButton4.setBackground(new java.awt.Color(153, 0, 0));
        jButton4.setFont(new java.awt.Font("Arial", 1, 12));
        jButton4.setIcon(new javax.swing.ImageIcon(getClass().getResource("resources/img/annotation.jpg"))); // NOI18N
        jButton4.setToolTipText("Annotation of the selection");
        jButton4.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton4ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton4);

        jButton10.setBackground(new java.awt.Color(153, 0, 0));
        jButton10.setFont(new java.awt.Font("Arial", 1, 12));
        jButton10.setIcon(new javax.swing.ImageIcon(getClass().getResource("resources/img/TextOnTop3.jpg"))); // NOI18N
        jButton10.setToolTipText("Text on top for the selection");
        jButton10.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton10ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton10);

        jButtonTransparency.setBackground(new java.awt.Color(153, 0, 0));
        jButtonTransparency.setFont(new java.awt.Font("Arial", 1, 12));
        jButtonTransparency.setIcon(new javax.swing.ImageIcon(getClass().getResource("resources/img/transparency.jpg"))); // NOI18N
        jButtonTransparency.setToolTipText("Display settings");
        jButtonTransparency.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonTransparencyActionPerformed(evt);
            }
        });
        jToolBar1.add(jButtonTransparency);

        jLabel21.setFont(new java.awt.Font("Arial", 1, 12));
        jLabel21.setText("       ");
        jToolBar1.add(jLabel21);

        jButton8.setBackground(new java.awt.Color(153, 0, 0));
        jButton8.setFont(new java.awt.Font("Arial", 1, 12));
        jButton8.setIcon(new javax.swing.ImageIcon(getClass().getResource("resources/img/tripod.jpg"))); // NOI18N
        jButton8.setToolTipText("Tripod");
        jButton8.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton8ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton8);

        jButton9.setBackground(new java.awt.Color(153, 0, 0));
        jButton9.setFont(new java.awt.Font("Arial", 1, 12));
        jButton9.setIcon(new javax.swing.ImageIcon(getClass().getResource("resources/img/resetView.jpg"))); // NOI18N
        jButton9.setToolTipText("Reset view");
        jButton9.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton9ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton9);

        ButtonSnapshot.setBackground(new java.awt.Color(153, 0, 0));
        ButtonSnapshot.setFont(new java.awt.Font("Arial", 1, 12));
        ButtonSnapshot.setIcon(new javax.swing.ImageIcon(getClass().getResource("resources/img/snapshot-1.jpg"))); // NOI18N
        ButtonSnapshot.setToolTipText("Snapshot");
        ButtonSnapshot.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ButtonSnapshotActionPerformed(evt);
            }
        });
        jToolBar1.add(ButtonSnapshot);

        jLabel2.setFont(new java.awt.Font("Arial", 1, 12));
        jLabel2.setText("       ");
        jToolBar1.add(jLabel2);

        jButton3.setBackground(new java.awt.Color(153, 0, 0));
        jButton3.setFont(new java.awt.Font("Arial", 1, 12));
        jButton3.setForeground(new java.awt.Color(255, 255, 255));
        jButton3.setIcon(new javax.swing.ImageIcon(getClass().getResource("resources/img/blink.jpg"))); // NOI18N
        jButton3.setToolTipText("Select blinking cells");
        jButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton3ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton3);

        jButton5.setBackground(new java.awt.Color(153, 0, 0));
        jButton5.setFont(new java.awt.Font("Arial", 1, 12));
        jButton5.setForeground(new java.awt.Color(255, 255, 255));
        jButton5.setIcon(new javax.swing.ImageIcon(getClass().getResource("resources/img/unblink.jpg"))); // NOI18N
        jButton5.setToolTipText("Unselect blinking cells");
        jButton5.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton5ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton5);

        jButton6.setBackground(new java.awt.Color(153, 0, 0));
        jButton6.setFont(new java.awt.Font("Arial", 1, 12));
        jButton6.setIcon(new javax.swing.ImageIcon(getClass().getResource("resources/img/DefaultCells.jpg"))); // NOI18N
        jButton6.setToolTipText("Stop blinking");
        jButton6.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton6ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton6);

        jButton2.setBackground(new java.awt.Color(153, 0, 0));
        jButton2.setFont(new java.awt.Font("Arial", 1, 12));
        jButton2.setForeground(new java.awt.Color(255, 255, 255));
        jButton2.setIcon(new javax.swing.ImageIcon(getClass().getResource("resources/img/stopBlink.jpg"))); // NOI18N
        jButton2.setToolTipText("Reset selection");
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton2);

        getContentPane().add(jToolBar1, java.awt.BorderLayout.NORTH);

        menuBar.setBackground(new java.awt.Color(153, 0, 0));
        menuBar.setBorder(null);
        menuBar.setFont(new java.awt.Font("Impact", 1, 12));

        fileMenu.setBackground(new java.awt.Color(153, 0, 0));
        fileMenu.setText("File");
        fileMenu.setFont(new java.awt.Font("Arial", 1, 12));

        ChangeList.setFont(new java.awt.Font("Arial", 1, 12));
        ChangeList.setForeground(new java.awt.Color(153, 0, 0));
        ChangeList.setText("Change embryo list");
        ChangeList.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ChangeListActionPerformed(evt);
            }
        });
        fileMenu.add(ChangeList);

        openMenuItem.setFont(new java.awt.Font("Arial", 1, 12));
        openMenuItem.setForeground(new java.awt.Color(153, 0, 0));
        openMenuItem.setText("Open 3D file");
        openMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                openMenuItemActionPerformed(evt);
            }
        });
        fileMenu.add(openMenuItem);

        exitMenuItem.setFont(new java.awt.Font("Arial", 1, 12));
        exitMenuItem.setForeground(new java.awt.Color(153, 0, 0));
        exitMenuItem.setText("Exit");
        exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitMenuItemActionPerformed(evt);
            }
        });
        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        EmbryoJMenu.setBackground(new java.awt.Color(153, 0, 0));
        EmbryoJMenu.setText("Embryos");
        EmbryoJMenu.setFont(new java.awt.Font("Arial", 1, 12));
        menuBar.add(EmbryoJMenu);

        AppearanceMenu.setBackground(new java.awt.Color(153, 0, 0));
        AppearanceMenu.setText("Appearance");
        AppearanceMenu.setFont(new java.awt.Font("Arial", 1, 12));

        MaterialMenuItem.setFont(new java.awt.Font("Arial", 1, 12));
        MaterialMenuItem.setForeground(new java.awt.Color(153, 0, 0));
        MaterialMenuItem.setText("Display settings");
        MaterialMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                MaterialMenuItemActionPerformed(evt);
            }
        });
        AppearanceMenu.add(MaterialMenuItem);

        jMenu1.setForeground(new java.awt.Color(153, 0, 0));
        jMenu1.setText("Coloring mode");

        buttonGroup3.add(jRadioButtonMenuItem12);
        jRadioButtonMenuItem12.setFont(new java.awt.Font("Arial", 1, 12));
        jRadioButtonMenuItem12.setForeground(new java.awt.Color(153, 0, 0));
        jRadioButtonMenuItem12.setSelected(true);
        jRadioButtonMenuItem12.setText("Diffuse color");
        jRadioButtonMenuItem12.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonMenuItem12ActionPerformed(evt);
            }
        });
        jMenu1.add(jRadioButtonMenuItem12);

        buttonGroup3.add(jRadioButtonMenuItem1);
        jRadioButtonMenuItem1.setFont(new java.awt.Font("Arial", 1, 12));
        jRadioButtonMenuItem1.setForeground(new java.awt.Color(153, 0, 0));
        jRadioButtonMenuItem1.setText("Ambiant color");
        jRadioButtonMenuItem1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonMenuItem1ActionPerformed(evt);
            }
        });
        jMenu1.add(jRadioButtonMenuItem1);

        buttonGroup3.add(jRadioButtonMenuItem13);
        jRadioButtonMenuItem13.setFont(new java.awt.Font("Arial", 1, 12));
        jRadioButtonMenuItem13.setForeground(new java.awt.Color(153, 0, 0));
        jRadioButtonMenuItem13.setText("Emissive color");
        jRadioButtonMenuItem13.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonMenuItem13ActionPerformed(evt);
            }
        });
        jMenu1.add(jRadioButtonMenuItem13);

        AppearanceMenu.add(jMenu1);

        aliasingRadio.setFont(new java.awt.Font("Arial", 1, 12));
        aliasingRadio.setForeground(new java.awt.Color(153, 0, 0));
        aliasingRadio.setText("Antialiasing");
        aliasingRadio.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                aliasingRadioActionPerformed(evt);
            }
        });
        AppearanceMenu.add(aliasingRadio);

        menuBar.add(AppearanceMenu);

        toolsMenu.setBackground(new java.awt.Color(153, 0, 0));
        toolsMenu.setText("Tools");
        toolsMenu.setFont(new java.awt.Font("Arial", 1, 12));

        MenuItemSnapshot.setFont(new java.awt.Font("Arial", 1, 12));
        MenuItemSnapshot.setForeground(new java.awt.Color(153, 0, 0));
        MenuItemSnapshot.setText("Snapshot");
        MenuItemSnapshot.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                MenuItemSnapshotActionPerformed(evt);
            }
        });
        toolsMenu.add(MenuItemSnapshot);

        MenuItemResetView.setFont(new java.awt.Font("Arial", 1, 12));
        MenuItemResetView.setForeground(new java.awt.Color(153, 0, 0));
        MenuItemResetView.setText("Reset view");
        MenuItemResetView.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                MenuItemResetViewActionPerformed(evt);
            }
        });
        toolsMenu.add(MenuItemResetView);

        tripodRadioButton.setFont(new java.awt.Font("Arial", 1, 12));
        tripodRadioButton.setForeground(new java.awt.Color(153, 0, 0));
        tripodRadioButton.setText("Show tripod");
        tripodRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                tripodRadioButtonActionPerformed(evt);
            }
        });
        toolsMenu.add(tripodRadioButton);

        menuBar.add(toolsMenu);

        helpMenu.setBackground(new java.awt.Color(153, 0, 0));
        helpMenu.setText("Help");
        helpMenu.setFont(new java.awt.Font("Arial", 1, 12));

        contentsMenuItem.setFont(new java.awt.Font("Arial", 1, 12));
        contentsMenuItem.setForeground(new java.awt.Color(153, 0, 0));
        contentsMenuItem.setText("Contents");
        contentsMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                contentsMenuItemActionPerformed(evt);
            }
        });
        helpMenu.add(contentsMenuItem);

        aboutMenuItem.setFont(new java.awt.Font("Arial", 1, 12));
        aboutMenuItem.setForeground(new java.awt.Color(153, 0, 0));
        aboutMenuItem.setText("About");
        aboutMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                aboutMenuItemActionPerformed(evt);
            }
        });
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);

        setJMenuBar(menuBar);

        pack();
    }// </editor-fold>//GEN-END:initComponents
////////////////////////////////////////////////////////////////////////////////////////////////////
//Change embryo list
    private void ChangeListActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ChangeListActionPerformed

        try{
            JFileChooser chooser = new JFileChooser(lastPath);

            int returnVal = chooser.showOpenDialog(this);

            if (returnVal == JFileChooser.APPROVE_OPTION) {
                String file=chooser.getSelectedFile().toString();
                if ( file.length() >0 ){
                    lastPath=file;
                    embryolist(chooser.getSelectedFile().getParent(), chooser.getSelectedFile().getName());
                }
            }
        }catch(Exception e){;}

    }//GEN-LAST:event_ChangeListActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
//Hierarchical query simulation
public void select_recursif(String start,Hashtable objs,Vector vec) throws SQLException
{
    Statement psmt = null;
    ResultSet rs = null;
    psmt=con.createStatement();
    rs=psmt.executeQuery("SELECT * FROM \"ANAT_ENTITY\" WHERE \"CHILD_OF\"="+start);
    while (rs.next())
    {
    try{
    String name = rs.getString(1);
    cell shape=(cell) objs.get(name);
    // Fix: issue with upper case obj
    if(shape == null) {
        shape = (cell) objs.get(name.toLowerCase());
    }

    if(! vec.contains(name)) shape.setSelected(true, jButton15.getBackground(), shape.isSelected());
    vec.add(name);
    }
    catch(NullPointerException e ){;}

    //recursivity
    select_recursif(start,objs,vec);
    }
}
////////////////////////////////////////////////////////////////////////////////////////////////////
// select objects correponding to the ids
    private void selectchecked1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_selectchecked1ActionPerformed

        try{
            String ids=txt.getText();
            Pattern p = Pattern.compile(" ");
            Matcher m = p.matcher(ids);
            ids=m.replaceAll("");

            p = Pattern.compile("\n");
            m = p.matcher(ids);
            ids=m.replaceAll("");

            if(ids.length()>0){

                //Connection to the database
                DriverManager.registerDriver(new org.postgresql.Driver());
                String url = VirtualEmbryo.database;
                String userID = VirtualEmbryo.DBlog;
                String pass = VirtualEmbryo.DBpass;

                con = DriverManager.getConnection(url, userID, pass);
                Statement s;
                ResultSet result=null;
                ///////////

                String[] allids=ids.split(":");
                Hashtable objs=embryon.getObj();

                if(hiera.isSelected())
                {
                    Vector vec=new Vector(); //avoid doublons
                    for(int a=0; a< allids.length; ++a){
                    	select_recursif(allids[a],objs,vec);
                    }
                }
                else
                {
                    Set keys = objs.keySet();
                    Iterator ite = keys.iterator();

                    while (ite.hasNext())
                    {
                        Object objtmp = ite.next();
                        cell node = (cell) objs.get(objtmp);
                        String anatid=node.getTreeRef().getID();

                        for(int a=0; a< allids.length; ++a)
                        {
                            if( allids[a].equals(anatid) )
                            {
                                node.setSelected(true, jButton15.getBackground(), node.isSelected());
                            }
                        }
                    }
                }
            }

        }
        catch(Exception e){;}
    }//GEN-LAST:event_selectchecked1ActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void jButton15ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton15ActionPerformed
        java.awt.Color bgColor = JColorChooser.showDialog(this,"Choose object Color",new java.awt.Color(153,0,0));
        if (bgColor != null){
            jButton15.setBackground(bgColor);
        }
    }//GEN-LAST:event_jButton15ActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void jButton71ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton71ActionPerformed
        txt.setText("");
    }//GEN-LAST:event_jButton71ActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void MaterialMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_MaterialMenuItemActionPerformed
        materialPanel.display(true);
    }//GEN-LAST:event_MaterialMenuItemActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
//Load embryo (on an event ...)
    public void loadEmbryo(java.awt.event.ActionEvent evt){

        EmbryoMenu info = (EmbryoMenu) evt.getSource();
        String name=info.getName();
        String file=info.getFile();
        String id  =info.getId();
        NavEmbryoPos=info.getPosition();

        try{
            view=embryon.getViewPlatform();
        }catch(Exception e){;}

        String prevstage=this.devstage;
        try{
            lineagelist=embryon.getSelectedCells();
        }catch(Exception e){;}

        try{
            this.clear();
        }catch(Exception e){;}

        try{
            embryon=null;
            embryon=new embryo(file);
            this.setTitle("3D Virtual Embryo - "+name);

            devstage=id;

            ScrollPaneSlicer.setViewportView(embryon.addSlicer());
            ScrollPaneSurf.setViewportView( new Biometry(embryon.getObj()) );
            jScrollPaneNeighbor.setViewportView( new BiometryNeighbor(embryon.getObj()) );
            jScrollPaneDistance.setViewportView( new BiometryDistance(embryon.getObj()) );
            jScrollPaneSphericity.setViewportView( new BiometrySphericity(embryon.getObj()) );
            jScrollPaneFate.setViewportView( new Fate(embryon.getObj()) );

            ScrollPaneQuery.setViewportView( new Query(embryon.getObj()) );
            ScrollPaneQuery2.setViewportView( new QuerySelection(embryon.getObj()) );
            ScrollPaneQuery3.setViewportView( new QueryFate(embryon.getObj()) );
            jScrollPane1.setViewportView( new competence(embryon.getObj()) );
            jScrollPane2.setViewportView( new induction(embryon.getObj()) );

            CanvasPanel.add( embryon );
            CanvasPanel.updateUI();

            treePanel = new DynamicTree();
            initTree( embryon.getObj() );

            System.gc();

        }catch(Exception e){System.out.println(e.getMessage());};

        try{
            //select descendant or ancestor cells
            if(lineagelist.size()>0)
                embryon.setSelectedCells(lineagelist,prevstage);
        }catch(Exception e){;}

    }
////////////////////////////////////////////////////////////////////////////////////////////////////
//Load embryo with a file
    public void loadEmbryo(String name,String file,String id,int position){

        NavEmbryoPos=position;

        try{
            view=embryon.getViewPlatform();
        }catch(Exception e){;}

        String prevstage=this.devstage;
        try{
            lineagelist=embryon.getSelectedCells();
        }catch(Exception e){;}

        try{
            this.clear();
        }catch(Exception e){;}


        try{

            embryon=null;
            embryon=new embryo(file);
            this.setTitle("3D Virtual Embryo - "+name);
            devstage=id;

            ScrollPaneSlicer.setViewportView(embryon.addSlicer());
            ScrollPaneSurf.setViewportView( new Biometry(embryon.getObj()) );
            jScrollPaneNeighbor.setViewportView( new BiometryNeighbor(embryon.getObj()) );
            jScrollPaneDistance.setViewportView( new BiometryDistance(embryon.getObj()) );
            jScrollPaneSphericity.setViewportView( new BiometrySphericity(embryon.getObj()) );
            jScrollPaneFate.setViewportView( new Fate(embryon.getObj()) );

            ScrollPaneQuery.setViewportView( new Query(embryon.getObj()) );
            ScrollPaneQuery2.setViewportView( new QuerySelection(embryon.getObj()) );
            ScrollPaneQuery3.setViewportView( new QueryFate(embryon.getObj()) );
            jScrollPane1.setViewportView( new competence(embryon.getObj()) );
            jScrollPane2.setViewportView( new induction(embryon.getObj()) );

            CanvasPanel.add( embryon );
            CanvasPanel.updateUI();

            treePanel = new DynamicTree();
            initTree( embryon.getObj() );

            System.gc();


        }catch(Exception e){;};

        try{
            //select descendant or ancestor cells
            if(lineagelist.size()>0)
                embryon.setSelectedCells(lineagelist,prevstage);
        }catch(Exception e){;}
    }
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void jButtonnextActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonnextActionPerformed
        try{
            EmbryoMenu emb = (EmbryoMenu) EmbryoJMenu.getItem(NavEmbryoPos +1);
            loadEmbryo(emb.getName(),emb.getFile(),emb.getId(),emb.getPosition());
        }catch(Exception e){;};
    }//GEN-LAST:event_jButtonnextActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void jButtonprevActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonprevActionPerformed
        try{
            EmbryoMenu emb = (EmbryoMenu) EmbryoJMenu.getItem(NavEmbryoPos -1);
            loadEmbryo(emb.getName(),emb.getFile(),emb.getId(),emb.getPosition());
        }catch(Exception e){;};
    }//GEN-LAST:event_jButtonprevActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void jRadioButtonMenuItem13ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonMenuItem13ActionPerformed
        embryon.setColorMode(3); //emissive color
    }//GEN-LAST:event_jRadioButtonMenuItem13ActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void jRadioButtonMenuItem1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonMenuItem1ActionPerformed
        embryon.setColorMode(2); //ambiant color
    }//GEN-LAST:event_jRadioButtonMenuItem1ActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void jRadioButtonMenuItem12ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonMenuItem12ActionPerformed
        embryon.setColorMode(1); //diffuse color
    }//GEN-LAST:event_jRadioButtonMenuItem12ActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void aliasingRadioActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_aliasingRadioActionPerformed
        embryon.setAntialiased(aliasingRadio.isSelected());
    }//GEN-LAST:event_aliasingRadioActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void jButton12ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton12ActionPerformed
        try{
            if(materialPanel.isTopVisible()){
                embryon.setZbuffer(false);
                materialPanel.SetTopVisible(false);
            }else{
                embryon.setZbuffer(true);
                materialPanel.SetTopVisible(true);
            }
        }catch(NullPointerException e){;}
    }//GEN-LAST:event_jButton12ActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void jButton11ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton11ActionPerformed
        try{
            if(materialPanel.isTextVisible()){
                embryon.setTextVisible(false);
                materialPanel.SetTextVisible(false);
            }else{
                embryon.setTextVisible(true);
                materialPanel.SetTextVisible(true);
            }
        }catch(NullPointerException e){;}
    }//GEN-LAST:event_jButton11ActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void jButton10ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton10ActionPerformed
        try{
            if(materialPanel.isSelectedTopVisible()){
                embryon.setSelectedZbuffer(false);
                materialPanel.SetSelectedTopVisible(false);
            }else{
                embryon.setSelectedZbuffer(true);
                materialPanel.SetSelectedTopVisible(true);
            }
        }catch(NullPointerException e){;}
    }//GEN-LAST:event_jButton10ActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void MenuItemResetViewActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_MenuItemResetViewActionPerformed
        try{
            embryon.resetView();
        }catch(NullPointerException e){;}
    }//GEN-LAST:event_MenuItemResetViewActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void jButton9ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton9ActionPerformed
        try{
            embryon.resetView();
        }catch(NullPointerException e){;}
    }//GEN-LAST:event_jButton9ActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void openMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_openMenuItemActionPerformed

        try{
            this.clear();
        }catch(Exception e){;}

        try{
            JFileChooser chooser = new JFileChooser(lastPath);
            chooser.setFileView(new ImageFileView());
            chooser.addChoosableFileFilter(new OnlyOBJFiltering());
            int returnVal = chooser.showOpenDialog(this);

            if (returnVal == JFileChooser.APPROVE_OPTION) {
                String file=chooser.getSelectedFile().toString();
                if ( file.length() >0 ){
                    lastPath=file;
                    embryon=new embryo(file);


                    species ="";
                    devstage="";
                    this.setTitle("User object");

                    ScrollPaneSlicer.setViewportView(embryon.addSlicer());
                    ScrollPaneSurf.setViewportView( new Biometry(embryon.getObj()) );
                    jScrollPaneNeighbor.setViewportView( new BiometryNeighbor(embryon.getObj()) );
                    jScrollPaneDistance.setViewportView( new BiometryDistance(embryon.getObj()) );
                    jScrollPaneSphericity.setViewportView( new BiometrySphericity(embryon.getObj()) );
                    jScrollPaneFate.setViewportView( new Fate(embryon.getObj()) );

                    treePanel = new DynamicTree();
                    //initTree( embryon.getObj() );

                    ScrollPaneQuery.setViewportView( new Query(embryon.getObj()) );
                    ScrollPaneQuery2.setViewportView( new QuerySelection(embryon.getObj()) );
                    ScrollPaneQuery3.setViewportView( new QueryFate(embryon.getObj()) );
                    jScrollPane1.setViewportView( new competence(embryon.getObj()) );
                    jScrollPane2.setViewportView( new induction(embryon.getObj()) );

                    CanvasPanel.add( embryon );
                    CanvasPanel.updateUI();

                    treePanel = new DynamicTree();
                    initTree( embryon.getObj() );
                }
            }
        }catch(Exception e){;};

    }//GEN-LAST:event_openMenuItemActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void jButton6ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton6ActionPerformed
        try{
            embryon.stopBlinking();
        }catch(NullPointerException e){;}
    }//GEN-LAST:event_jButton6ActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
        try{
            embryon.reset();
        }catch(NullPointerException e){;}
    }//GEN-LAST:event_jButton2ActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void jButton5ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton5ActionPerformed
        try{
            embryon.getNotBlinking();
        }catch(NullPointerException e){;}
    }//GEN-LAST:event_jButton5ActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void jButton3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton3ActionPerformed
        try{
            embryon.getBlinking();
            embryon.stopBlinking();
        }catch(NullPointerException e){;}
    }//GEN-LAST:event_jButton3ActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void jButton4ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton4ActionPerformed
        try{
            if(materialPanel.isSelectedTextVisible()){
                embryon.setSelectedTextVisible(false);
                materialPanel.SetSelectedTextVisible(false);
            }else{
                embryon.setSelectedTextVisible(true);
                materialPanel.SetSelectedTextVisible(true);
            }
        }catch(NullPointerException e){;}
    }//GEN-LAST:event_jButton4ActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void jButtonTransparencyActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonTransparencyActionPerformed

        materialPanel.display(true);
    }//GEN-LAST:event_jButtonTransparencyActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void MenuItemSnapshotActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_MenuItemSnapshotActionPerformed
        try{
            embryon.snapshot();
        }catch(NullPointerException e){;}
    }//GEN-LAST:event_MenuItemSnapshotActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void jButton8ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton8ActionPerformed
        try{
            if( tripodRadioButton.isSelected() ){
                embryon.showTripod(false);
                tripodRadioButton.setSelected(false);
            }else{
                embryon.showTripod(true);
                tripodRadioButton.setSelected(true);
            }
        }catch(NullPointerException e){;}
    }//GEN-LAST:event_jButton8ActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void tripodRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_tripodRadioButtonActionPerformed
        try{
            if( tripodRadioButton.isSelected() ){
                embryon.showTripod(true);
            }else{
                embryon.showTripod(false);
            }
        }catch(NullPointerException e){;}
    }//GEN-LAST:event_tripodRadioButtonActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void checknoneActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_checknoneActionPerformed
        JTree tree=treePanel.getTree();
        for(int a=0; a<tree.getRowCount(); ++a){
            anatomyPart part= (anatomyPart) tree.getPathForRow(a).getLastPathComponent();
            part.setSelected(false);
        }
        tree.updateUI();
    }//GEN-LAST:event_checknoneActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void checkallActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_checkallActionPerformed
        JTree tree=treePanel.getTree();
        for(int a=0; a<tree.getRowCount(); ++a){
            anatomyPart part= (anatomyPart) tree.getPathForRow(a).getLastPathComponent();
            part.setSelected(true);
        }
        tree.updateUI();
    }//GEN-LAST:event_checkallActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void jButton7ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton7ActionPerformed
        JTree tree=treePanel.getTree();
        treePanel.expandAllObject();
        for(int a=0; a<tree.getRowCount(); ++a){
            anatomyPart part= (anatomyPart) tree.getPathForRow(a).getLastPathComponent();
            part.setSelected(false);
            part.setColor(new Color(153,0,0));
        }
        treePanel.collapseAllObject();
    }//GEN-LAST:event_jButton7ActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void selectinvertActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_selectinvertActionPerformed

        JTree tree=treePanel.getTree();
        java.util.Vector selection=new java.util.Vector();

        treePanel.expandAllObject();
        for(int a=0; a<tree.getRowCount(); ++a){
            if (! tree.isRowSelected(a) ){
                selection.add( new Integer(a) );
            }
        }

        int[] rows = new int[selection.size()];
        for(int a=0;a<selection.size();++a)
            rows[a]= selection.get(a).hashCode() ;

        tree.setSelectionRows( rows );
        tree.updateUI();

    }//GEN-LAST:event_selectinvertActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void selectcheckedActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_selectcheckedActionPerformed
        JTree tree=treePanel.getTree();
        java.util.Vector selection=new java.util.Vector();

        treePanel.expandAllObject();
        for(int a=0; a<tree.getRowCount(); ++a){
            anatomyPart part= (anatomyPart) tree.getPathForRow(a).getLastPathComponent();
            if(part.isSelected()){
              selection.add( new Integer(a) );
            }
        }

        int[] rows = new int[selection.size()];
        for(int a=0;a<selection.size();++a)
            rows[a]= selection.get(a).hashCode() ;

        tree.setSelectionRows( rows );
        tree.updateUI();
    }//GEN-LAST:event_selectcheckedActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed

        Color Couleur=ColorPick(new Color(153,0,0));
        if (Couleur != null){
            JTree tree=treePanel.getTree();
            int[] rows = tree.getSelectionRows();
            if(rows != null){
                for(int a=0; a<rows.length; ++a){
                    anatomyPart part= (anatomyPart) tree.getPathForRow(rows[a]).getLastPathComponent();
                    if( embryon.getObj().get( part.getName() )!=null  )
                        {
                        //change the node color only if ot corresponds to a 3d object
                        part.setColor(Couleur);
                        if(part.isSelected())
                            part.setSelected(true);  //Update Color on 3D
                    }
                }
                tree.updateUI();
            }
        }

    }//GEN-LAST:event_jButton1ActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void checkinvertActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_checkinvertActionPerformed

        JTree tree=treePanel.getTree();
        treePanel.expandAllObject();
        for(int a=0; a<tree.getRowCount(); ++a){
            anatomyPart part= (anatomyPart) tree.getPathForRow(a).getLastPathComponent();

            try{
                if(part.isSelected())
                    part.getShape().setSelected( false );
                else
                    part.getShape().setSelected( true );
            }catch(NullPointerException e){;}
        }
        tree.updateUI();

    }//GEN-LAST:event_checkinvertActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void checkActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_checkActionPerformed
        JTree tree=treePanel.getTree();
        int[] rows = tree.getSelectionRows();
        if(rows != null){
            for(int a=0; a<rows.length; ++a){
                anatomyPart part= (anatomyPart) tree.getPathForRow(rows[a]).getLastPathComponent();
                part.setSelected(true);
            }
            tree.updateUI();
        }
    }//GEN-LAST:event_checkActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void collallActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_collallActionPerformed
        treePanel.collapseAllObject();
    }//GEN-LAST:event_collallActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void expallActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_expallActionPerformed
        treePanel.expandAllObject();
    }//GEN-LAST:event_expallActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void exitMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitMenuItemActionPerformed
        System.exit(0);
    }//GEN-LAST:event_exitMenuItemActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void ButtonSnapshotActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ButtonSnapshotActionPerformed
        try{
            embryon.snapshot();
        }catch(Exception e){;}
    }//GEN-LAST:event_ButtonSnapshotActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void contentsMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_contentsMenuItemActionPerformed
        try{
            BrowserLauncher.openURL( VirtualEmbryo.url+"virtual_embryo.php" );
        }catch(Exception e){;}
    }//GEN-LAST:event_contentsMenuItemActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void aboutMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_aboutMenuItemActionPerformed
        JOptionPane.showMessageDialog(this,"3D Virtual Embryo v1.3\n          Copyright © 2003-2013\nAuthors: \n          Olivier Tassy\n          Patrick Lemaire\nCNRS - CRBM\n1919, route de Mende\nF-34293 Montpellier cedex 5\nFrance\n\nThis software is distributed under the terms\nof the GNU Lesser General Public License.\n");
    }//GEN-LAST:event_aboutMenuItemActionPerformed
////////////////////////////////////////////////////////////////////////////////////////////////////
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        //Exit
        try{
            con.close();
        }
        catch(SQLException e){;}
        finally{
            System.exit(0);
        }
    }//GEN-LAST:event_exitForm
////////////////////////////////////////////////////////////////////////////////////////////////////
    private Color ColorPick(Color defColor){
        //Allow the user to choose objects colors
        Color bgColor = JColorChooser.showDialog(this,"Choose objects Color",defColor);
        return(bgColor);
    }
////////////////////////////////////////////////////////////////////////////////////////////////////
    /*public static void clear(){
        embryon.destroy();
        embryon=null;
        CanvasPanel.remove(embryon);
    }*/

    public static void clear(){
    embryon.destroy();
    CanvasPanel.remove(embryon);
    embryon=null;
    }


////////////////////////////////////////////////////////////////////////////////////////////////////
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenu AppearanceMenu;
    static javax.swing.JPanel CanvasPanel;
    public javax.swing.JMenu EmbryoJMenu;
    private javax.swing.JMenuItem MaterialMenuItem;
    private javax.swing.JMenuItem MenuItemResetView;
    private javax.swing.JMenuItem MenuItemSnapshot;
    private javax.swing.JScrollPane ScrollPaneQuery;
    private javax.swing.JScrollPane ScrollPaneQuery2;
    private javax.swing.JScrollPane ScrollPaneQuery3;
    private javax.swing.JScrollPane ScrollPaneSelection;
    private javax.swing.JScrollPane ScrollPaneSelection1;
    private javax.swing.JScrollPane ScrollPaneSlicer;
    private javax.swing.JScrollPane ScrollPaneSurf;
    private javax.swing.JMenuItem aboutMenuItem;
    private javax.swing.JRadioButtonMenuItem aliasingRadio;
    private javax.swing.ButtonGroup buttonGroup1;
    private javax.swing.ButtonGroup buttonGroup2;
    private javax.swing.ButtonGroup buttonGroup3;
    private javax.swing.ButtonGroup buttonGroup4;
    private javax.swing.JMenuItem contentsMenuItem;
    private javax.swing.JMenu fileMenu;
    private javax.swing.JMenu helpMenu;
    private javax.swing.JCheckBox hiera;
    private javax.swing.JButton jButton11;
    private javax.swing.JButton jButton12;
    private javax.swing.JButton jButton13;
    private javax.swing.JButton jButton15;
    private javax.swing.JButton jButtonnext;
    private javax.swing.JButton jButtonprev;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JRadioButtonMenuItem jRadioButtonMenuItem1;
    private javax.swing.JRadioButtonMenuItem jRadioButtonMenuItem12;
    private javax.swing.JRadioButtonMenuItem jRadioButtonMenuItem13;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPaneDistance;
    private javax.swing.JScrollPane jScrollPaneFate;
    private javax.swing.JScrollPane jScrollPaneNeighbor;
    private javax.swing.JScrollPane jScrollPaneSphericity;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JTabbedPane jTabbedPane3;
    private javax.swing.JTabbedPane jTabbedPane5;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JMenu toolsMenu;
    private javax.swing.JRadioButtonMenuItem tripodRadioButton;
    private javax.swing.JTextArea txt;
    // End of variables declaration//GEN-END:variables
////////////////////////////////////////////////////////////////////////////////////////////////////
//Hierarchical query
    public void select_recursif2(String start,int level,Hashtable hash,Hashtable objs,String devStage) throws SQLException
    {
    	Statement psmt = null;
        ResultSet rs = null;

        psmt=con.createStatement();


        if (!start.equals(""))
        {
            rs=psmt.executeQuery("SELECT \"ANAT_ID\", \"NAME\", \"TYPE\", \"CHILD_OF\" FROM \"ANAT_ENTITY\" WHERE  \"DEV_STAGE_ID\"= '"+devStage+"' and  \"CHILD_OF\"="+start);
        }
        else
        {
            rs=psmt.executeQuery("SELECT \"ANAT_ID\", \"NAME\", \"TYPE\", \"CHILD_OF\" FROM \"ANAT_ENTITY\" WHERE  \"DEV_STAGE_ID\"= '"+devStage+"' and  \"CHILD_OF\" is null");
        }

        if (psmt.getFetchSize()!=1) level++;
        if (psmt.getFetchSize()==1) level--;

        while (rs.next())
        {
             String id, name, type, childof;
             id=rs.getString(1);

             if(rs.wasNull()){
                 id=null;
             }

             name=rs.getString(2);

             if(rs.wasNull()){
                 name=null;
             }

             if(name.equals("No expression"))
                 continue;

             type=rs.getString(3);
             if(rs.wasNull()){
                 type=null;
             }

             childof=rs.getString(4);
             if(rs.wasNull()){
                 childof=null;
             }

             //shape to be associated to the tree branch;
             cell shape = (cell) objs.get(name);
             // Fix: issue with upper case obj
             if(shape == null) {
                 shape = (cell) objs.get(name.toLowerCase());
             }

             if(level==1){
                 treePanel.setRoot(name, id, "organism", shape);
             }else{
                 anatomyPart p = treePanel.addObject( (anatomyPart) hash.get(childof), name, type, id, shape);
                 hash.put(id,p);
             }

             treePanel.setPreferredSize(new java.awt.Dimension(0,0));
             treePanel.setMinimumSize(new java.awt.Dimension(0,0));

             ScrollPaneSelection.setViewportView(treePanel);

             //Recursivity
             select_recursif2(id,level,hash,objs,devStage);

        }

    }
////////////////////////////////////////////////////////////////////////////////////////////////////
//Display anatomical tree for the selected embryo
   public void initTree(Hashtable objs){

        if(devstage.equals("")){
            defaultTree(objs);
            return;
        }

        try{
            DriverManager.registerDriver(new org.postgresql.Driver());
            String url = database;
            String userID = DBlog;
            String pass= DBpass;
            con = DriverManager.getConnection(url, userID, pass);
            con.setAutoCommit(false);

           Statement pstmt = null;
            ResultSet rs = null;

            pstmt=con.createStatement();

            java.util.Hashtable hash=new java.util.Hashtable();

            //Hierarchical query
            select_recursif2("",0,hash,objs,devstage);

            con.close();

        }catch( SQLException e){

            if( con !=null){
                try{ con.rollback();}
                catch (SQLException ev){;}
            }

            JOptionPane diag2=new JOptionPane();
            diag2.showMessageDialog(new javax.swing.JFrame(),"Error: "+ e.getMessage());

            defaultTree(objs);

        }

        finally {
            if( con !=null){
                try {con.close();}
                catch (SQLException e) {;}
                }
        }
    }
////////////////////////////////////////////////////////////////////////////////////////////////////
//Display a default tree wich contains only blastomers
    private void defaultTree(Hashtable objs){

        //generates a default tree made of 3d object names.
        treePanel.setRoot("Whole embryo", "0");
        anatomyPart root=(anatomyPart) treePanel.getrootNode();

        Set keys = embryon.getObj().keySet();
        Iterator ite = keys.iterator();
        while (ite.hasNext()){
            Object name=ite.next();
            //Look for the shape associated to the tree line
            cell shape = (cell) objs.get(name);
            anatomyPart p = treePanel.addObject( root, name, "cell", "0", shape);
        }
        treePanel.setPreferredSize(new java.awt.Dimension(0,0));
        treePanel.setMinimumSize(new java.awt.Dimension(0,0));
        ScrollPaneSelection.setViewportView(treePanel);

    }
////////////////////////////////////////////////////////////////////////////////////////////////////
    public static Hashtable getObj(){
        return embryon.getObj();
    }
////////////////////////////////////////////////////////////////////////////////////////////////////
//Main function
    public static void main(String args[]) {

        //Loading
        new VirtualEmbryo().setVisible(true);

    }
////////////////////////////////////////////////////////////////////////////////////////////////////


}
